@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_process -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -23,11 +23,14 @@ try:
 except ImportError:
     fcntl = None
 
+from zope.interface import implements
+
 from twisted.python import log, failure
 from twisted.python.util import switchUID
 from twisted.internet import fdesc, abstract, error
 from twisted.internet.main import CONNECTION_LOST, CONNECTION_DONE
 from twisted.internet._baseprocess import BaseProcess
+from twisted.internet.interfaces import IProcessTransport
 
 # Some people were importing this, which is incorrect, just keeping it
 # here for backwards compatibility:
@@ -334,7 +337,13 @@ class _BaseProcess(BaseProcess, object):
             signalID = getattr(signal, 'SIG%s' % (signalID,))
         if self.pid is None:
             raise ProcessExitedAlready()
-        os.kill(self.pid, signalID)
+        try:
+            os.kill(self.pid, signalID)
+        except OSError, e:
+            if e.errno == errno.ESRCH:
+                raise ProcessExitedAlready()
+            else:
+                raise
 
 
     def _resetSignalDisposition(self):
@@ -463,6 +472,129 @@ class _BaseProcess(BaseProcess, object):
         return "<%s pid=%s status=%s>" % (self.__class__.__name__,
                                           self.pid, self.status)
 
+
+class _FDDetector(object):
+    """
+    This class contains the logic necessary to decide which of the available
+    system techniques should be used to detect the open file descriptors for
+    the current process. The chosen technique gets monkey-patched into the
+    _listOpenFDs method of this class so that the detection only needs to occur
+    once.
+
+    @ivars listdir: The implementation of listdir to use. This gets overwritten
+        by the test cases.
+    @ivars getpid: The implementation of getpid to use, returns the PID of the
+        running process.
+    @ivars openfile: The implementation of open() to use, by default the Python
+        builtin.
+    """
+    # So that we can unit test this
+    listdir = os.listdir
+    getpid = os.getpid
+    openfile = open
+
+    def __init__(self):
+        self._implementations = [
+            self._procFDImplementation, self._devFDImplementation,
+            self._fallbackFDImplementation]
+
+
+    def _listOpenFDs(self):
+        """
+        Return an iterable of file descriptors which I{may} be open in this
+        process.
+
+        This will try to return the fewest possible descriptors without missing
+        any.
+        """
+        self._listOpenFDs = self._getImplementation()
+        return self._listOpenFDs()
+
+
+    def _getImplementation(self):
+        """
+        Pick a method which gives correct results for C{_listOpenFDs} in this
+        runtime environment.
+
+        This involves a lot of very platform-specific checks, some of which may
+        be relatively expensive.  Therefore the returned method should be saved
+        and re-used, rather than always calling this method to determine what it
+        is.
+
+        See the implementation for the details of how a method is selected.
+        """
+        for impl in self._implementations:
+            try:
+                before = impl()
+            except:
+                continue
+            try:
+                fp = self.openfile("/dev/null", "r")
+                after = impl()
+            finally:
+                fp.close()
+            if before != after:
+                return impl
+        # If no implementation can detect the newly opened file above, then just
+        # return the last one.  The last one should therefore always be one
+        # which makes a simple static guess which includes all possible open
+        # file descriptors, but perhaps also many other values which do not
+        # correspond to file descriptors.  For example, the scheme implemented
+        # by _fallbackFDImplementation is suitable to be the last entry.
+        return impl
+
+
+    def _devFDImplementation(self):
+        """
+        Simple implementation for systems where /dev/fd actually works.
+        See: http://www.freebsd.org/cgi/man.cgi?fdescfs
+        """
+        dname = "/dev/fd"
+        result = [int(fd) for fd in self.listdir(dname)]
+        return result
+
+
+    def _procFDImplementation(self):
+        """
+        Simple implementation for systems where /proc/pid/fd exists (we assume
+        it works).
+        """
+        dname = "/proc/%d/fd" % (self.getpid(),)
+        return [int(fd) for fd in self.listdir(dname)]
+
+
+    def _fallbackFDImplementation(self):
+        """
+        Fallback implementation where either the resource module can inform us
+        about the upper bound of how many FDs to expect, or where we just guess
+        a constant maximum if there is no resource module.
+
+        All possible file descriptors from 0 to that upper bound are returned
+        with no attempt to exclude invalid file descriptor values.
+        """
+        try:
+            import resource
+        except ImportError:
+            maxfds = 1024
+        else:
+            # OS-X reports 9223372036854775808. That's a lot of fds to close.
+            # OS-X should get the /dev/fd implementation instead, so mostly
+            # this check probably isn't necessary.
+            maxfds = min(1024, resource.getrlimit(resource.RLIMIT_NOFILE)[1])
+        return range(maxfds)
+
+
+
+detector = _FDDetector()
+
+def _listOpenFDs():
+    """
+    Use the global detector object to figure out which FD implementation to
+    use.
+    """
+    return detector._listOpenFDs()
+
+
 class Process(_BaseProcess):
     """
     An operating-system Process.
@@ -476,6 +608,7 @@ class Process(_BaseProcess):
     code is not cross-platform. (also, windows can only select
     on sockets...)
     """
+    implements(IProcessTransport)
 
     debug = False
     debug_child = False
@@ -601,7 +734,8 @@ class Process(_BaseProcess):
         This is accomplished in two steps::
 
             1. close all file descriptors that aren't values of fdmap.  This
-               means 0 .. maxfds.
+               means 0 .. maxfds (or just the open fds within that range, if
+               the platform supports '/proc/<pid>/fd').
 
             2. for each childFD::
 
@@ -625,16 +759,7 @@ class Process(_BaseProcess):
             errfd.write("starting _setupChild\n")
 
         destList = fdmap.values()
-        try:
-            import resource
-            maxfds = resource.getrlimit(resource.RLIMIT_NOFILE)[1] + 1
-            # OS-X reports 9223372036854775808. That's a lot of fds to close
-            if maxfds > 1024:
-                maxfds = 1024
-        except:
-            maxfds = 256
-
-        for fd in xrange(maxfds):
+        for fd in _listOpenFDs():
             if fd in destList:
                 continue
             if debug and fd == errfd.fileno():
@@ -799,10 +924,13 @@ class Process(_BaseProcess):
         _BaseProcess.maybeCallProcessEnded(self)
 
 
+
 class PTYProcess(abstract.FileDescriptor, _BaseProcess):
     """
     An operating-system Process that uses PTY support.
     """
+    implements(IProcessTransport)
+
     status = -1
     pid = None
 
@@ -885,11 +1013,12 @@ class PTYProcess(abstract.FileDescriptor, _BaseProcess):
         os.dup2(slavefd, 1) # stdout
         os.dup2(slavefd, 2) # stderr
 
-        for fd in xrange(3, 256):
-            try:
-                os.close(fd)
-            except:
-                pass
+        for fd in _listOpenFDs():
+            if fd > 2:
+                try:
+                    os.close(fd)
+                except:
+                    pass
 
         self._resetSignalDisposition()
 
